USE [<db_name,,AdventureWorks2012>];
GO

--SET NOCOUNT ON;
CREATE TABLE #rebuild
	(
	[schema] NVARCHAR(128)
	, [table] NVARCHAR(128)
	, [index] NVARCHAR(128)
	, [index_id] INT
--	, [type_desc] NVARCHAR(60)
	, [rows] BIGINT
	, [rebuild_index_command] NVARCHAR(4000)
--	, [fill_factor] TINYINT
	);

CREATE TABLE #process_order
(
	rn bigint, 
	[object_id] bigint,   	                        
	[the_table] NVARCHAR(128), 
	[total_io] bigint, 
	sum_seeks bigint, 
	sum_scans bigint, 
	sum_lookups bigint, 
	sum_updates bigint
)  


INSERT INTO [#process_order] (rn, [object_id], [the_table], [total_io], [sum_seeks], [sum_scans], [sum_lookups], [sum_updates])
SELECT ROW_NUMBER() OVER(ORDER BY (SUM(user_seeks) + SUM([user_scans]) + SUM([user_lookups]) + SUM([user_updates])) DESC) AS rn, [object_id], object_name(object_id) AS [the_table], (SUM(user_seeks) + SUM([user_scans]) + SUM([user_lookups]) + SUM([user_updates])) AS [total_io], SUM(user_seeks) AS sum_seeks, SUM([user_scans]) AS sum_scans, SUM([user_lookups]) AS sum_lookups, SUM([user_updates]) AS sum_updates
FROM sys.[dm_db_index_usage_stats]
WHERE db_name([database_id]) = '<db_name,,AdventureWorks2012>' 
--	AND [index_id] = 1
GROUP BY [object_id] --object_name(object_id)
--ORDER BY 1 ASC


------------------
DECLARE @schema NVARCHAR(128), @table NVARCHAR(128), @index NVARCHAR(128), @ordinal SMALLINT, @column NVARCHAR(128), @rows BIGINT, @is_unique BIT, @is_desc BIT, @column_list NVARCHAR(128), @index_id INT, @type_desc NVARCHAR(60), @fill_factor VARCHAR(3);
DECLARE @old_schema NVARCHAR(128), @old_table NVARCHAR(128), @old_index NVARCHAR(128), @old_ordinal SMALLINT, @old_column NVARCHAR(128), @old_rows BIGINT, @old_is_unique BIT, @old_is_desc BIT, @old_index_id INT, @old_type_desc NVARCHAR(60), @old_fill_factor VARCHAR(3);
DECLARE @sql NVARCHAR(4000);

SELECT @column_list = NULL;
SELECT @old_ordinal = 1
-------------------




--------------------


DECLARE table_cursor CURSOR FOR 
	SELECT SCHEMA_NAME(sO.[schema_id]) AS [schema], sO.[name] AS [table_name], sI.[name] AS [index_name], sIC.key_ordinal, sAC.[name] AS [column_name], sP.[rows], sI.[is_unique], sIC.[is_descending_key]  
		, sI.[index_id], sI.[type_desc], CAST(sI.[fill_factor] AS VARCHAR(3)) AS [fill_factor]
	FROM sys.[objects] sO
		INNER JOIN sys.[indexes] sI ON [sO].[object_id] = [sI].[object_id]
		INNER JOIN sys.[index_columns] sIC ON [sI].[index_id] = [sIC].[index_id] AND [sI].[object_id] = [sIC].[object_id]
		INNER JOIN sys.[all_columns] sAC ON [sIC].[column_id] = [sAC].[column_id] AND [sIC].[object_id] = [sAC].[object_id]
		INNER JOIN sys.filegroups sFG ON sI.[data_space_id] = sFG.[data_space_id]
		INNER JOIN sys.[partitions] sP ON sI.object_id = sP.object_id AND sI.index_id = sP.index_id 
		LEFT JOIN 
		(
			SELECT DISTINCT AC.[object_id], AC.[name] AS [column_name]
			FROM sys.[all_columns] AC 
				INNER JOIN sys.types T
					ON [AC].[system_type_id] = [T].[system_type_id]
			WHERE (AC.[max_length] = (-1) OR T.[name] IN ('text', 'ntext', 'image'))
		) BLOBS ON sI.[object_id] = [BLOBS].[object_id]
	WHERE sO.[is_ms_shipped] = 0
		AND sFG.[name] = 'PRIMARY'
		AND sI.[type_desc] = 'CLUSTERED'
		AND sO.[name] <> 'sysdiagrams'
		AND [BLOBS].[object_id] IS NULL
--AND sP.[rows] < 100000		
	ORDER BY sO.[name], sIC.[key_ordinal]

OPEN table_cursor

FETCH NEXT FROM table_cursor INTO @schema, @table, @index, @ordinal, @column,  @rows, @is_unique, @is_desc, @index_id, @type_desc, @fill_factor

WHILE @@FETCH_STATUS = 0
BEGIN
    IF @column_list IS NULL	--First record in set
		BEGIN
			IF @is_desc = 1	  
				BEGIN
					SELECT @column_list = '[' + @column + '] DESC'
				END
				
			ELSE
				BEGIN
					SELECT @column_list = '[' + @column + ']'
				END
		END 
	ELSE
		BEGIN
			IF @ordinal > @old_ordinal --Multiple Columns in Clustered Index; same index still
				BEGIN
					IF @is_desc = 1				 
						BEGIN 
							SELECT @column_list = @column_list + ', [' + @column + '] DESC'
						END
						
					ELSE
						BEGIN 
							SELECT @column_list = @column_list + ', [' + @column + ']'
						END
				END
			ELSE	-- This is a new Index
				BEGIN
					IF @old_is_unique = 1 
						BEGIN
							SELECT @sql = 'CREATE UNIQUE CLUSTERED INDEX [' + @old_index + '] ON ['+ @old_schema + '].[' + @old_table + '] (' + @column_list + ') WITH (DROP_EXISTING=ON, ONLINE=ON, MAXDOP = 6, FILLFACTOR=' + @fill_factor + ') ON [<filegoup,,USER_DATA>];'
						END
					ELSE 
						BEGIN
							SELECT @sql = 'CREATE CLUSTERED INDEX [' + @old_index + '] ON ['+ @old_schema + '].[' + @old_table + '] (' + @column_list + ') WITH (DROP_EXISTING=ON, ONLINE=ON, MAXDOP = 6, FILLFACTOR=' + @fill_factor + ') ON [<filegoup,,USER_DATA>];'
						END
					
					INSERT INTO [#rebuild]
					        (
								 [schema]
							   , [table]
							   , [index]
							   , [index_id]
							   , [rows]
							   , [rebuild_index_command]
					        )
					VALUES  (@old_schema, @old_table, @old_index, @old_index_id, @old_rows, @sql)
					
					IF @is_desc = 1 
						BEGIN 
							SELECT @column_list = '[' + @column + '] DESC'
						END 
					ELSE 
						BEGIN
							SELECT @column_list = '[' + @column + ']'
						END
				END
		END

	SELECT @old_schema = @schema
	SELECT @old_table = @table
	SELECT @old_index = @index
	SELECT @old_column = @column
	SELECT @old_rows = @rows
	SELECT @old_is_unique = @is_unique
	SELECT @old_index_id = @index_id
	SELECT @old_type_desc = @type_desc
	SELECT @old_fill_factor = @fill_factor

	FETCH NEXT FROM table_cursor INTO @schema, @table, @index, @ordinal, @column,  @rows, @is_unique, @is_desc, @index_id, @type_desc, @fill_factor
END 
CLOSE table_cursor;
DEALLOCATE table_cursor;

--Capture Final Record----------------------------
IF @old_is_unique = 1 
	BEGIN
		SELECT @sql = 'CREATE UNIQUE CLUSTERED INDEX [' + @old_index + '] ON ['+ @old_schema + '].[' + @old_table + '] (' + @column_list + ') WITH (DROP_EXISTING=ON, ONLINE=ON, MAXDOP = 6, FILLFACTOR=' + @fill_factor + ') ON [<filegoup,,USER_DATA>];'
	END
ELSE 
	BEGIN
		SELECT @sql = 'CREATE CLUSTERED INDEX [' + @old_index + '] ON ['+ @old_schema + '].[' + @old_table + '] (' + @column_list + ') WITH (DROP_EXISTING=ON, ONLINE=ON, MAXDOP = 6, FILLFACTOR=' + @fill_factor + ') ON [<filegoup,,USER_DATA>];'
	END
					
INSERT INTO [#rebuild]
	(
		[schema]
		, [table]
		, [index]
		, [index_id]
		, [rows]
		, [rebuild_index_command]
	)
VALUES  (@old_schema, @old_table, @old_index, @old_index_id, @old_rows, @sql)
					
IF @is_desc = 1 
	BEGIN 
		SELECT @column_list = @column + ' DESC'
	END 
ELSE 
	BEGIN
		SELECT @column_list = @column
	END
--------------------------------------------------------


SELECT RB.* , PO.*	--, i.[object_id] AS foo
FROM [#rebuild] RB
	LEFT JOIN #process_order PO ON RB.[table] = PO.[the_table]
--ORDER BY PO.rn, RB.[rows] DESC	--, [schema], [table];
ORDER BY RB.[rows] DESC, PO.rn

DROP TABLE [#rebuild];
DROP TABLE #process_order;

-----------------