--================================================
--  BASELINING RAM AND IO OVER TOGETHER
--================================================

DECLARE @start_date datetime
DECLARE @days_sampled as BIGINT
DECLARE @buffer_size_mb BIGINT	--Buffer Pool Size

SELECT @start_date = create_date from sys.databases where name = 'tempdb'
SELECT @days_sampled = datediff(d, @start_date, getdate()) 

IF @days_sampled = 0 
	SELECT @days_sampled = 1

SELECT @buffer_size_mb = (bpool_visible/1024*8) FROM sys.dm_os_sys_info;

SELECT physical_memory_mb = (SELECT physical_memory_in_bytes/1024/1024 FROM sys.dm_os_sys_info)
	, max_server_memory_mb = (SELECT bpool_commit_target/1024*8 FROM sys.dm_os_sys_info)
	, @buffer_size_mb AS buffer_pool_size_mb 
	, (SUM(ioFS.[num_of_reads]))/@days_sampled as num_of_reads_per_day
	, (SUM(ioFS.[num_of_writes]))/@days_sampled as num_of_writes_per_day
	, (SUM(ioFS.num_of_bytes_read))/1024/1024/1024/@days_sampled as GB_read_per_day
	, (SUM(ioFS.num_of_bytes_written))/1024/1024/1024/@days_sampled as GB_written_per_day
	, ((SUM(ioFS.num_of_bytes_read))/1024/1024/@days_sampled)/@buffer_size_mb as est_complete_cycles_daily
	, @start_date as first_date_in_sample
FROM sys.dm_io_virtual_file_stats(NULL,NULL) ioFS 
GROUP BY ioFS.sample_ms
GO




--=============================
-- VOLUME LEVEL BASELINE
--=============================
DECLARE @start_date datetime

SELECT @start_date = create_date 
FROM sys.databases WHERE name = 'tempdb'

SELECT LEFT(MF.physical_name,1) AS volume
	, SUM([ioFS].[num_of_reads]) AS read_count
	, SUM([ioFS].[num_of_bytes_read])/1024/1024 AS read_mb
	, CASE SUM([ioFS].[num_of_reads])
		WHEN 0 THEN 0
		ELSE CAST(SUM([ioFS].[io_stall_read_ms])/(SUM([ioFS].[num_of_reads]))/1.0 AS DECIMAL(10,1)) 
	END AS [avg_read_stall_ms]
	, SUM([ioFS].[num_of_writes]) AS write_count
	, SUM([ioFS].[num_of_bytes_written])/1024/1024 AS write_mb
	, CASE SUM([ioFS].[num_of_writes])
		WHEN 0 THEN 0
		ELSE CAST(SUM([ioFS].[io_stall_write_ms])/(SUM([ioFS].[num_of_writes]))/1.0 AS DECIMAL(10,1))
	END AS [avg_write_stall_ms]
	, SUM([ioFS].[size_on_disk_bytes])/1024 AS file_size_mb
	, datediff(d, @start_date, getdate()) as days_sampled
	, getdate() as date_stamp
FROM sys.dm_io_virtual_file_stats(NULL,NULL) ioFS 
	INNER JOIN sys.master_files MF 
		ON ioFS.database_id = MF.database_id 
			AND ioFS.file_id = MF.file_id
GROUP BY LEFT(MF.physical_name,1)
ORDER BY 1;
GO

