SET NOCOUNT ON
GO

--==================================================================
--DETERMINE WHAT INFORMATION FOR WAITS HAS BEEN PERSISTED TO DISK--
--==================================================================
SELECT DISTINCT [wait_id], [date_stamp] 
FROM iDBA.[dbo].[wait_history]
ORDER BY [wait_id] DESC;




--==================================================================
--DETERMINE LAST TWO CAPTURE INFORMATION FOR WAITS HAS BEEN PERSISTED TO DISK--
--==================================================================
SELECT TOP 2 [wait_id], [date_stamp] 
FROM iDBA.[dbo].[wait_history]
GROUP BY wait_id, date_stamp
ORDER BY [wait_id] DESC;
GO





--==================================================================
--DETERMINE WAITS BETWEEN TWO CAPTURE POINTS PERSISTED TO DISK
--==================================================================
IF EXISTS (SELECT name FROM tempdb..sysobjects WHERE name LIKE '%#periodic_waits%')
	BEGIN
		DROP TABLE #periodic_waits;
	END	
GO

CREATE TABLE #periodic_waits
	(
		rn tinyint, 
		--[wait_id] [int] ,
		[start_date_stamp] [datetime] ,
		[end_date_stamp] datetime,
		[wait_type] [varchar](1000) ,
		[wait_time_s] [bigint] ,
		[signal_wait_time_s] [bigint] ,
		[resource_wait_time_s] [bigint] ,
		[waiting_task_count] [bigint] ,
		[avg_resource_wait_time_ms] DECIMAL (5,1) ,
		[pct] [decimal](5, 2)
--		[running_pct] [decimal](5, 2)  
	);
  
WITH first_waits AS 
(
SELECT  [date_stamp] AS [start_date_stamp]
	, [wait_type]
      , [wait_time_s]
      , [signal_wait_time_s]
      , [resource_wait_time_s]
      , [waiting_task_count]
      , [avg_resource_wait_time_ms]
      --, [pct]
      --, [running_pct] 
FROM iDBA.[dbo].[wait_history] 
WHERE [wait_id] = <wait_id_start,int,(SELECT TOP 1 first_wait.[wait_id] FROM (SELECT DISTINCT TOP 2 [wait_id] FROM iDBA.[dbo].[wait_history] ORDER BY [wait_id] DESC) AS first_wait ORDER BY first_wait.wait_id ASC)>
)

--Where both a start and end value exists for same wait type:
INSERT INTO #periodic_waits
	(
		rn
		, start_date_stamp
		, end_date_stamp
		, wait_type
        , wait_time_s
        , signal_wait_time_s
        , resource_wait_time_s
        , waiting_task_count
        , avg_resource_wait_time_ms
        , pct
		)
	SELECT 
		ROW_NUMBER() OVER(ORDER BY ((last_waits.wait_time_s - first_waits.wait_time_s) - (last_waits.[signal_wait_time_s] - first_waits.[signal_wait_time_s])) DESC) AS rn
		, first_waits.start_date_stamp
		, last_waits.date_stamp as end_date_stamp
		, last_waits.wait_type
		, last_waits.wait_time_s - first_waits.wait_time_s AS wait_time_s
		, last_waits.[signal_wait_time_s] - first_waits.[signal_wait_time_s] AS [signal_wait_time_s]
		, last_waits.[resource_wait_time_s] - first_waits.[resource_wait_time_s] AS [resource_wait_time_s]
		, last_waits.[waiting_task_count] - first_waits.[waiting_task_count] AS [waiting_task_count]
		, CASE last_waits.[waiting_task_count] - first_waits.[waiting_task_count]
			WHEN 0 THEN 0
			ELSE CAST(1000.0 * (last_waits.[resource_wait_time_s] - first_waits.[resource_wait_time_s])/ (last_waits.[waiting_task_count] - first_waits.[waiting_task_count]) AS DECIMAL (5,1))
		END AS avg_wait_time_ms
		, CAST(100. * (last_waits.wait_time_s - first_waits.wait_time_s) / SUM(last_waits.wait_time_s - first_waits.wait_time_s) OVER() AS DECIMAL(5,2)) AS pct
--		, CAST(SUM(W2.pct) AS DECIMAL(5, 2)) AS running_pct
	FROM iDBA.[dbo].[wait_history] last_waits 
		INNER JOIN first_waits ON last_waits.wait_type = first_waits.wait_type
	WHERE last_waits.wait_id = <wait_id_end,int,(SELECT TOP 1 wait_id FROM iDBA.[dbo].[wait_history]  ORDER BY wait_id desc)>
	  


SELECT 
	W1.start_date_stamp,
	W1.end_date_stamp,
	W1.wait_type,
	CAST(W1.wait_time_s AS DECIMAL(10, 0)) AS wait_time_s,
	CAST(W1.signal_wait_time_s AS DECIMAL(10, 0)) AS signal_wait_time_s,
	CAST(W1.resource_wait_time_s AS DECIMAL(10, 0)) AS resource_wait_time_s,
	W1.waiting_task_count,
	W1.avg_resource_wait_time_ms,
	CAST(W1.pct AS DECIMAL(5, 2)) AS pct,
	CAST(SUM(W2.pct) AS DECIMAL(5, 2)) AS running_pct
FROM #periodic_waits AS W1
	INNER JOIN #periodic_waits AS W2 ON W2.rn <= W1.rn
GROUP BY W1.rn, 
	W1.start_date_stamp,
	W1.end_date_stamp,
	W1.wait_type, 
	W1.wait_time_s,
	W1.signal_wait_time_s,
	W1.resource_wait_time_s,
	W1.avg_resource_wait_time_ms, 
	W1.waiting_task_count,
	W1.pct
HAVING SUM(W2.pct) - W1.pct < 95; -- percentage threshold 
GO

IF EXISTS (SELECT name FROM tempdb..sysobjects WHERE name LIKE '%#periodic_waits%')
	BEGIN
		DROP TABLE #periodic_waits;
	END	
